package d11;

import static fi.jyu.mit.graphics.FunctionMapRR.funMin;
import static java.lang.Math.PI;
import fi.jyu.mit.graphics.FunctionMapRR.*;
import fi.jyu.mit.graphics.FunctionRR.*;

public class TehtavaT6 {


    /**
     * Rajapinta kaikille funktiolle R->R
     */
    public interface FunktioRR {
        /**
         * @param x piste jossa lasketaan
         * @return funktion arvo pisteessä
         */
        public double f(double x);
    }

    /**
     * Luokka sini-funktiolla
     */
    public static class SinFun implements FunktioRR {
        @Override
        public double f(double x) {
            return Math.sin(x);
        }
    }

    /**
     * Luokka 2-asteen polynomille
     */
    public static class P2 implements FunktioRR {
        private double a;
        private double b;
        private double c;

        /**
         * Alustetaan polynomiksi x^2
         */
        public P2() {
            a = 1;
        }


        /**
         * Alustetaan polynomi kertoimilla
         * @param a x^2 kerroin
         * @param b x:n kerroin
         * @param c vakiotermi
         */
        public P2(double a, double b, double c) {
            this.a = a;
            this.b = b;
            this.c = c;
        }

        @Override
        public double f(double x) {
            return a * x * x + b * x + c;
        }
    }


    /**
     * Integroidaan sin(x) välillä x1-x2
     * @param f integroitva funktio-olio
     * @param x1 alkuarvo
     * @param x2 loppuarvo
     * @param tiheys monellako askeleella
     * @return likiarvo integraalille
     */
    public static double integroi(FunktioRR f, double x1, double x2, int tiheys) {
        double summa = 0;
        double dx = (x2 - x1) / tiheys;
        for (double x = x1 + dx / 2; x < x2; x += dx)
            summa += f.f(x) * dx;
        return summa;
    }

    /**
     * @param f integroitva funktio-olio
     * @param x1 alkuarvo
     * @param x2 loppuarvo
     * @param dx vaihteluväli monellako askeleella
     * @return integraalin pienimmän y:n arvon
     * @example <pre name="test">
     * SinFun sinf = new SinFun();
     * double x1=-5,x2=5,dx=0.5;
     * String.format("%.2f", funMin(sinf, x1, x2, dx)) === "-1.00"
     * dx = 1.0;
     * String.format("%.3f", funMin(sinf, x1, x2, dx)) === "-0.959"
     * </pre>
     */
    public static double funMin(FunktioRR f, double x1, double x2, double dx) {
        double minimi = f.f(x2);
        for (double x = x1; x < x2; x += dx)
            minimi = Math.min(minimi, f.f(x));
        return minimi;
    }

    /**
     * @param f integroitva funktio-olio
     * @param x1 alkuarvo
     * @param x2 loppuarvo
     * @param dx vaihteluväli monellako askeleella
     * @return integraalin suurimman y:n arvon
     */
    public static double funMax(FunktioRR f, double x1, double x2, double dx) {
        double maksimi = f.f(x2);
        for (double x = x1; x < x2; x += dx)
            maksimi = Math.max(maksimi, f.f(x));
        return maksimi;
    }

    /**
     * @param args ei käytössä
     */
    public static void main(String[] args) {
        SinFun sinf = new SinFun();
        P2 p2 = new P2(1, 2, -3); // x^2 +2x -3
        FunktioRR exp = new FunktioRR() {
            @Override
            public double f(double x) { return Math.exp(x); }
        };

        double ala = integroi(sinf, 0, PI, 10000);
        System.out.printf("%17.15f%n", ala);

        ala = integroi(p2, 0, 3, 100);
        System.out.printf("%17.15f%n", ala);

        ala = integroi(exp,0,3,1000);
        System.out.printf("%17.15f%n",ala);

        ala = integroi(new FunktioRR() {
            @Override
            public double f(double x) { return x*x + 1; }
        },0,3,1000);
        System.out.printf("%17.15f%n",ala);

        ala = integroi((x) -> x*x + 1 ,0,3,1000);
        System.out.printf("%17.15f%n",ala);

        ala = integroi(x -> 4 ,0,3,1000);
        System.out.printf("%17.15f%n",ala);

        double y1 = funMin(sinf, -5, 5, 0.5);
        System.out.println("\n y1 arvo:");
        System.out.printf("%17.15f%n",y1);

        double y2 = funMax(sinf, -5, 5, 0.5);
        System.out.println("\n y1 arvo:");
        System.out.printf("%17.15f%n",y2);
    }
}
