package d8;

import java.util.Arrays;

/**
 * Luokkien alut mäkihyppyä varten.  Huom!  jotakin voi vielä puuttua :-)
 * Poista rivit @SuppressWarnings("unused")
 *
 * @author Joonas Yli-Saari
 * @version 1.0, 10.03.2025
 */
public class Makihyppy {

    /**
     * Luokka yhtä mäkihypyn kierrosta varten
     */
    public static class Kierros {
        private final static int TUOMAREITA = 5;
        private double pituus; // hyppyjen pituudet metreinä
        private final double[] tuomarit = new double[TUOMAREITA];// tuomaripisteet

        /**
         * Kierroksen alustaminen
         *
         * @example <pre name="test">
         * Makihyppy.Kierros k = new Makihyppy.Kierros();
         * k.getPisteet() ~~~ 0.0
         * </pre>
         */
        public Kierros() {
            this.pituus = 0.0;
            Arrays.fill(tuomarit, 0.0);
        }

        public void setPituus(double pituus) {
            this.pituus = pituus;
        }

        public void setTuomari(int tuomari, double pisteet) {
            this.tuomarit[tuomari-1] = pisteet;
        }

        /**
         * @return palauttaa tuomareidein antamat kokonaispisteet
         */
        public double getPisteet() {
            double K_PISTE = 60.0;
            double METRIKERROIN = 1.0;
            double summa = d7.TehtavaT8.summaHuonoinJaParasPois(tuomarit);
            return summa > 0 ? METRIKERROIN * pituus - K_PISTE + summa : 0;
        }

        @Override
        public String toString() {
            StringBuilder builder = new StringBuilder();
            builder.append(this.pituus + " m. Tuomarit: ");
            for (int i = 0; i < TUOMAREITA; i++) {
                builder.append(this.tuomarit[i]).append(" ");
            }
            builder.append(" = " + this.getPisteet() + " pistettä.");
            return builder.toString();
        }
    }

    /**
     * Luokka mäkihypyn yhden kilpailijan tulosta varten.
     * Sisältää monta kierosta.
     */
    public static class Tulos {
        private final static int KIERROKSIA = 2;
        private final Kierros[] kierros = new Kierros[KIERROKSIA];

        /**
         * Tuloksen alustaminen
         */
        public Tulos() {
            kierros[0] = new Kierros();
            kierros[1] = new Kierros();
        }

        public void setPituus(int kierros, double pituus) {
            this.kierros[kierros - 1].setPituus(pituus);
        }

        public void setTuomari(int kierros, int tuomari, double pisteet) {
            this.kierros[kierros - 1].setTuomari(tuomari, pisteet);
        }

        /**
         * @return palauttaa kierrosten yhteis pistemäärän
         */
        public double getPisteet() {
            return this.kierros[0].getPisteet() + this.kierros[1].getPisteet();
        }

        public String toString() {
            StringBuilder builder = new StringBuilder();
            for (int i = 0; i < kierros.length; i++) {
                builder.append("Kierros " + (i+1) + " " + kierros[i]).append('\n');
            }
            builder.append("Yhteensä: " + this.getPisteet() + " pistettä. \n");
            return builder.toString();
        }

    }


    /**
     * Luokka yhtä mäkihypyn kilpailijaa varten.
     * Sisältää mm. tuloksen.
     */
    public static class Kilpailija {
        private String nimi;
        private int nro;
        private final Tulos tulos = new Tulos();

        /**
         * Kilpailijan alustaminen
         *
         * @param nimi kilpailijan nimi
         * @param nro  kilpailijan kilpailunumero
         */
        public Kilpailija(String nimi, int nro) {
            this.nimi = nimi;
            this.nro = nro;
        }

        /**
         * Asettaa määritellylle kierrokselle pituuden
         *
         * @param kierros 1 tai 2 riippuen kummalle kierrokselle pituus asetetaan
         * @param pituus  asetettava pituus
         */
        public void setPituus(int kierros, int pituus) {
            this.tulos.setPituus(kierros, pituus);
        }

        public void setTuomari(int kierros, int tuomari, double pisteet) {
            this.tulos.setTuomari(kierros, tuomari, pisteet);
        }

        public double getPisteet() {
            return this.tulos.getPisteet();
        }

        public int compareTo(Kilpailija p) {
            if (this.getPisteet() > p.getPisteet()) {
                return 1;
            } else if (this.getPisteet() < p.getPisteet()) {
                return -1;
            } else {
                return 0;
            }
        }

        @Override
        public String toString() {
            StringBuilder builder = new StringBuilder();
            builder.append(this.nro + ": " + this.nimi + "\n");
            builder.append(this.tulos);
            return builder.toString();
        }
    }

    /**
     * Tyhjä aliohjelma kilpailua varten
     */
    public void kisa() {
        Kilpailija toni = new Kilpailija("Toni", 3);
        Kilpailija matti = new Kilpailija("Matti", 7);
        System.out.println(toni);
        System.out.println(matti);

        toni.setPituus(1, 107);       // 1. kierros, 107 m
        toni.setPituus(2, 100);       // 2. kierros, 100 m
        toni.setTuomari(2, 1, 19.0);   // 2. kierros, 1. tuomari, 19 p
        toni.setTuomari(2, 2, 18.0);   // 2. kierros, 2. tuomari, 18 p
        toni.setTuomari(2, 3, 19.5);
        toni.setTuomari(2, 4, 18.0);
        toni.setTuomari(2, 5, 20.0);

        matti.setPituus(1, 125);
        matti.setTuomari(1, 1, 20.0);
        matti.setTuomari(1, 2, 20.0);
        matti.setTuomari(1, 3, 20.0);
        matti.setTuomari(1, 4, 20.0);
        matti.setPituus(2, 109);
        matti.setTuomari(2, 1, 20.0);
        matti.setTuomari(2, 2, 20.0);
        matti.setTuomari(2, 3, 20.0);
        matti.setTuomari(2, 4, 20.0);

        System.out.println(toni);
        System.out.println(matti);

        if (matti.compareTo(toni) > 0) System.out.println("Matti voitti");
    }


    public static void main(String[] args) {
        Makihyppy kisa = new Makihyppy();
        kisa.kisa();
    }
}